/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions;

import org.wso2.registry.*;
import org.wso2.registry.web.actions.utils.VersionPath;
import org.wso2.registry.web.actions.utils.Permission;
import org.wso2.registry.secure.RegistryUserManager;
import org.wso2.registry.secure.SecureRegistry;
import org.wso2.usermanager.Realm;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

public class ResourceDetailsAction extends AbstractRegistryAction {

    private String path;
    private String name;
    private String author;
    private String description;
    private Date createdOn;
    private Date lastModified;
    private String mediaType;
    private Properties properties;

    private List tags = new ArrayList();
    private List comments = new ArrayList();
    private float averageRating;
    private List versionPaths = new ArrayList();
    private List userPermissions = new ArrayList();

    private String parentPath;

    /**
     * Used to populate the users in the userPermissions section
     */
    private List userNames = new ArrayList();

    public String execute(HttpServletRequest request) throws Exception {

        setRequest(request);
        
        Registry registry = getRegistry();

        // if path is not set, we assume that root "/" is requested
        if (path == null || path.length() == 0) {
            path = RegistryConstants.ROOT_PATH;
        }

        if (path.equals(RegistryConstants.ROOT_PATH)) {
            name = "root";
        } else {
            String[] parts = path.split(RegistryConstants.PATH_SEPARATOR);
            name = parts[parts.length - 1];
        }

        Resource resource = registry.get(path);

        if (resource.getAuthorUserName() != null) {
            this.author = resource.getAuthorUserName();
        }

        if (resource.getDescription() != null) {
            this.description = resource.getDescription();
        }

        this.path = resource.getPath();
        this.createdOn = resource.getCreatedTime();
        this.lastModified = resource.getLastModified();
        this.mediaType = resource.getMediaType();
        this.averageRating = registry.getAverageRating(resource.getPath());

        Tag[] t = registry.getTags(resource.getPath());
        for (int i = 0; i < t.length; i++) {
            tags.add(t[i]);
        }

        Comment[] c = registry.getComments(resource.getPath());
        for (int i = 0; i < c.length; i++) {
            comments.add(c[i]);
        }

        String[] versions = registry.getVersions(resource.getPath());
        for (int i = 0; i < versions.length; i++) {
            VersionPath versionPath = new VersionPath();
            versionPath.setCompleteVersionPath(versions[i]);
            if (versions[i].indexOf("?v=") > 0) {
                long versionNumber = new Long(versions[i].split("\\?v=")[1]).longValue();
                versionPath.setVersionNumber(versionNumber);
            }
            versionPaths.add(versionPath);
        }

        // construct the parent path
        if (path.equals(RegistryConstants.ROOT_PATH)) {
            parentPath = "root";
        } else {
            String formattedPath = path;
            if (path.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                formattedPath = path.substring(
                        0, path.length() - RegistryConstants.PATH_SEPARATOR.length());
            }

            if (formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR) == 0) {
                parentPath = RegistryConstants.ROOT_PATH;
            } else {
                parentPath = formattedPath.substring(
                        0, formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR));
            }
        }

        RegistryUserManager userManager = ((SecureRegistry) registry).getUserManager();
        String[] userArray = userManager.getAllUsers();
        for (int i = 0; i < userArray.length; i++) {
            userNames.add(userArray[i]);
        }

        Realm realm = userManager.getRealm();

        Map userPermissionMap = new HashMap();

        String[] raUsers = realm.getAuthorizer().getAllowedUsersForResource(path, ActionConstants.GET);
        for (int i = 0; i < raUsers.length; i++) {
            if (userPermissionMap.containsKey(raUsers[i])) {
                Permission permission = (Permission) userPermissionMap.get(raUsers[i]);
                permission.setReadAllow(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(raUsers[i]);
                permission.setReadAllow(true);
                userPermissionMap.put(raUsers[i], permission);
            }
        }

        String[] rdUsers = realm.getAuthorizer().getDeniedUsersForResource(path, ActionConstants.GET);
        for (int i = 0; i < rdUsers.length; i++) {
            if (userPermissionMap.containsKey(rdUsers[i])) {
                Permission permission = (Permission) userPermissionMap.get(rdUsers[i]);
                permission.setReadDeny(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(rdUsers[i]);
                permission.setReadDeny(true);
                userPermissionMap.put(rdUsers[i], permission);
            }
        }

        String[] waUsers = realm.getAuthorizer().getAllowedUsersForResource(path, ActionConstants.PUT);
        for (int i = 0; i < waUsers.length; i++) {
            if (userPermissionMap.containsKey(waUsers[i])) {
                Permission permission = (Permission) userPermissionMap.get(waUsers[i]);
                permission.setWriteAllow(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(waUsers[i]);
                permission.setWriteAllow(true);
                userPermissionMap.put(waUsers[i], permission);
            }
        }

        String[] wdUsers = realm.getAuthorizer().getDeniedUsersForResource(path, ActionConstants.PUT);
        for (int i = 0; i < wdUsers.length; i++) {
            if (userPermissionMap.containsKey(wdUsers[i])) {
                Permission permission = (Permission) userPermissionMap.get(wdUsers[i]);
                permission.setWriteDeny(true);
            } else {
                Permission permission = new Permission();
                permission.setUserName(wdUsers[i]);
                permission.setWriteDeny(true);
                userPermissionMap.put(wdUsers[i], permission);
            }
        }

        userPermissions = new ArrayList(userPermissionMap.values());
        
        return SUCCESS;
    }

    public String getPath() {
        return path;
    }

    public void setPath(String path) {
        this.path = path;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getAuthor() {
        return author;
    }

    public void setAuthor(String author) {
        this.author = author;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public Date getCreatedOn() {
        return createdOn;
    }

    public void setCreatedOn(Date createdOn) {
        this.createdOn = createdOn;
    }

    public Date getLastModified() {
        return lastModified;
    }

    public void setLastModified(Date lastModified) {
        this.lastModified = lastModified;
    }

    public String getMediaType() {
        return mediaType;
    }

    public void setMediaType(String mediaType) {
        this.mediaType = mediaType;
    }

    public Properties getProperties() {
        return properties;
    }

    public void setProperties(Properties properties) {
        this.properties = properties;
    }

    public float getAverageRating() {
        return averageRating;
    }

    public void setAverageRating(float averageRating) {
        this.averageRating = averageRating;
    }

    public List getTags() {
        return tags;
    }

    public void setTags(List tags) {
        this.tags = tags;
    }

    public List getComments() {
        return comments;
    }

    public void setComments(List comments) {
        this.comments = comments;
    }

    public List getVersionPaths() {
        return versionPaths;
    }

    public void setVersionPaths(List versionPaths) {
        this.versionPaths = versionPaths;
    }

    public String getParentPath() {
        return parentPath;
    }

    public void setParentPath(String parentPath) {
        this.parentPath = parentPath;
    }

    public List getUserNames() {
        return userNames;
    }

    public void setUserNames(List userNames) {
        this.userNames = userNames;
    }

    public List getUserPermissions() {
        return userPermissions;
    }

    public void setUserPermissions(List userPermissions) {
        this.userPermissions = userPermissions;
    }
}
